/**
 * Created by will on 10/1/15.
 */
//var models = require('./../models/index');
var fs = require('fs');
var path = require('path');
var winston = require('winston');
var logger = winston.loggers.get('default');
var transactionLogger = winston.loggers.get('transaction');

var masterItemDir = process.env.MASTER_FILE || '/DevTools/workspaces/fresenius/fresenius-m3-api/components/data/';
var countFiles = process.env.COUNT_FILE || '/Users/will/dev/fresenius/fresenius-m3-api/components/data/counts/';
var clinicFile = process.env.CLINIC_FILE || '/DevTools/workspaces/fresenius/fresenius-m3-api/components/data/';

var loadingCPPromise=null;
var loadingMIPromise=null;

if (!String.prototype.endsWith) {
    String.prototype.endsWith = function(searchString, position) {
        var subjectString = this.toString();
        if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
            position = subjectString.length;
        }
        position -= searchString.length;
        var lastIndex = subjectString.indexOf(searchString, position);
        return lastIndex !== -1 && lastIndex === position;
    };
}
if (!String.prototype.startsWith) {
    String.prototype.startsWith = function(searchString, position) {
        position = position || 0;
        return this.indexOf(searchString, position) === position;
    };
}

module.exports = function(app) {
    var models = app.get('models');
    fs.exists(masterItemDir, function (exists) {
        if (exists) {
            logger.debug("Started watching Files: " + masterItemDir);
            fs.watch(masterItemDir, function (event, filename) {
                if (filename != null) {
                    if (filename.indexOf("MEDSExtract") > -1) {
                        loadMasterItem(masterItemDir + filename);
                    } else if (filename.indexOf("CHIPS_LOCATOR") > -1) {
                        loadClinicProfile(masterItemDir + filename);
                    }
                }
            });
        }
    });

    fs.exists(countFiles, function (exists) {
        if (exists) {
            fs.watch(countFiles, function (event, filename) {
                fs.readFile(countFiles + filename, 'utf8', function (err, data) {
                    if (err) {
                        logger.error("err: " + err);
                    } else {
                        readCountData(data);
                        fs.rename(countFiles + filename, countFiles + 'archive/' + filename, function (err) {
                            if (err) {
                                logger.error("err: " + err);
                            }
                        });
                    }
                });
            });
        }
    });

    function loadCounts() {
        var countData = [];
        fs.readdir(countFiles, function (err, files) {
            if (err) {
                logger.error(err);
            } else {
                for (var i = 0; i < files.length; i++) {
                    fs.readFile(files[i], 'utf8', function (err, data) {
                        if (err) {
                            logger.error(err);
                        } else {
                            countData.push(data);
                        }
                    });
                }
            }
        });

        readCountData(countData);
    }

    function bulkCreateCountData(countDataObjects) {
        var promises = [];

        countDataObjects.forEach(function (countData) {
            promises.push(countData.save());
        });

        return models.sequelize.Promise.all(promises);
    }

    function loadMasterItem(file) {
        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                logger.error(err)
            } else {
                readMasterItems(data);
                archiveFile(file, data);
            }
        });
    }

    function archiveFile(file, data) {
        var archiveDir = path.join(masterItemDir, "archive");
        var archivePath = path.join(archiveDir, '/', path.basename(file));
        fs.exists(archiveDir, function (exists) {
            if (!exists) {
                fs.mkdir(archiveDir, function (err) {
                    if (err) {
                        logger.error(err);
                    } else {
                        fs.writeFile(archivePath, data, function (err) {
                            if (err) {
                                logger.error(err);
                            } else {
                                fs.unlink(file, function (err) {
                                    if (err) {
                                        logger.error(err);
                                    } else {
                                        logger.debug("moved and deleted file");
                                    }
                                });
                            }
                        });
                    }
                });
            } else {
                fs.writeFile(archivePath, data, function (err) {
                    if (err) {
                        logger.error(err);
                    } else {
                        fs.unlink(file, function (err) {
                            if (err) {
                                logger.error(err);
                            } else {
                                logger.debug("moved and deleted file");
                            }
                        });
                    }
                });
            }
        });
    }

    function loadClinicProfile(file) {
        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                logger.error(err);
            } else {
                readClinicProfiles(data);
                archiveFile(file, data);
            }
        });
    }

    function readCountData(data) {
        var lines = data.split(/\r?\n/);
        var counts = [];
        for (var i = 0; i < lines.length; i++) {
            var line = lines[i].split(',');
            if (line.length > 1) {
                var count = models.CountData.build({
                    timestamp: line[0],
                    clinicId: line[1],
                    sapMaterial: line[2],
                    vendorMaterial: line[3],
                    description: line[4],
                    count: line[5],
                    countType: line[6],
                    catType: line[7],
                    cntInd: line[8] == 'X'
                });
                counts.push(count);
            }
        }
        bulkCreateCountData(counts);
    }

    function readMasterItems(data) {
        if (loadingMIPromise && loadingMIPromise.toString() != data) {
            loadingMIPromise = null;
        }
        if (loadingMIPromise) {
            return loadingMIPromise;
        } else {
            // cache this promise so any other requests while this one is stil
            // in flight will use the same promise
            loadingMIPromise = new Promise(function (resolve, reject) {
                var allVendors = [];
                var lines = data.split(/\r?\n/);
                var masterItems = [];
                models.VendorList.findAll().then(function (vendors) {
                    if (vendors) allVendors = vendors;
                    for (var i = 0; i < lines.length; i++) {
                        var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
                        line = line || [];
                        if (line.length > 1) {
                            var testVendor = line[4].split('"').join('');
                            if (!testVendor) testVendor = "";
                            if (testVendor != "") {
                                //we need to convert names to id's
                                var fileVendorNames = testVendor.split(",");
                                testVendor = "";
                                var cnt = 0;
                                for (var fn = 0; fn < fileVendorNames.length; fn++) {
                                    var id = getVendorId(allVendors, fileVendorNames[fn]);
                                    if (id >= 0) {
                                        testVendor = testVendor + ((cnt > 0) ? "," : "") + id;
                                        cnt++;
                                    }
                                }
                            }
                            var masterItem = models.MasterItem.build({
                                sapMaterial: parseInt(line[0]),
                                description: line[1].split('"').join(''),
                                cntInd: line[2].split('"').join('') == 'X',
                                cntDescription: line[3].split('"').join(''),
                                medVendor: testVendor,
                                vendorMaterial: line[5].split('"').join(''),
                                conv: parseInt(line[6].split('"').join('')),
                                countType: line[7].split('"').join(''),
                                alternates: line[8].split('"').join(''),
                                dateModified: new Date()
                            });

                            masterItems.push(masterItem);
                        }
                    }
                    resolve(bulkUpsertMasterItems(masterItems));
                });
            });
            loadingMIPromise.__proto__.toString = function () {
                return data;
            };
            return loadingMIPromise;
        }
    }

    function readClinicProfiles(data) {
        if (loadingCPPromise && loadingCPPromise.toString() != data) {
            loadingCPPromise = null;
        }
        if (loadingCPPromise) {
            return loadingCPPromise;
        } else {
            // cache this promise so any other requests while this one is still
            // in flight will use the same promise
            loadingCPPromise = new Promise(function (resolve, reject) {
                var allVendors = [];
                var clinicProfiles = [];
                var strDefVendors = "";
                models.VendorList.findAll().then(function (vendors) {
                        if (vendors) allVendors = vendors;

                        for (var vl = 0; vl < allVendors.length; vl++) {
                            var dv = allVendors[vl];
                            if (dv.isDefault === true)
                                strDefVendors += "," + dv.id;
                        }
                        logger.debug("Default vendors for this load: " + strDefVendors);
                        clinicProfiles = buildClinicProfileObjects(data, allVendors, strDefVendors);
                        resolve(bulkUpsertClinicProfiles(clinicProfiles));
                    },
                    function (err) {
                        reject(err);
                    });

            });
            loadingCPPromise.__proto__.toString = function () {
                return data;
            };
            return loadingCPPromise;
        }
    }

    function getVendorId(allVendors, name) {
        var out = -1;
        if (allVendors && name) {
            for (var v = 0; v < allVendors.length; v++) {
                var vendor = allVendors[v];
                if (vendor.vendorId === name) {
                    out = vendor.id;
                    break;
                }
            }
        }
        return out;
    }

    function buildClinicProfileObjects(data, allVendors, strDefVendors) {
        var clinicProfiles = [];
        var lines = data.split(/\r?\n/);
        for (var i = 0; i < lines.length; i++) {
            var line = lines[i].split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
            if (line.length > 1) {
                var clinicId = line[0];
                if (clinicId.length == 4) {
                    clinicId = "." + clinicId + "BM"
                } else {
                    if (!clinicId.startsWith(".")) {
                        clinicId = "." + clinicId;
                    }
                }
                var testVendor = line[1].split('"').join('');
                if (!testVendor) {
                    testVendor = "";
                }
                if (testVendor != "") {
                    //we need to convert names to id's
                    var fileVendorNames = testVendor.split(",");
                    testVendor = "";
                    var cnt = 0;
                    for (var fn = 0; fn < fileVendorNames.length; fn++) {
                        var id = getVendorId(allVendors, fileVendorNames[fn]);
                        if (id >= 0) {
                            testVendor = testVendor + ((cnt > 0) ? "," : "") + id;
                            cnt++;
                        }
                    }
                    testVendor += strDefVendors;
                } else {
                    testVendor = strDefVendors.substr(1);
                }
                var clinicProfile = models.ClinicProfile.build({
                    clinicId: clinicId,
                    vendorList: testVendor,
                    mirceraVisible: line[2].split('"').join('') == 'y',
                    mirceraIo: line[3].split('"').join('') == 'y',
                    areanespVisible: line[4].split('"').join('') == 'y',
                    areanespIo: line[5].split('"').join('') == 'y',
                    epogenVisible: line[6].split('"').join('') == 'y',
                    epogenIo: line[7].split('"').join('') == 'y',
                    countInEach: line[8].split('"').join('') == 'y',
                    dateModified: new Date()
                });

                clinicProfiles.push(clinicProfile);
            }
        }
        return clinicProfiles;
    }


//fnd the record or create it
//if "deleted", update anyway, but leave the deleted flag on  (to skip altogether, remove the paranoid:false option)
    function upsertMasterItem(masterItem) {
        logger.debug("Trying " + masterItem.sapMaterial + ":" + masterItem.vendorMaterial);
        return models.MasterItem.findOrCreate({
            where: {
                sapMaterial: masterItem.sapMaterial,
                vendorMaterial: masterItem.vendorMaterial
            }, raw: true, paranoid: false
        })
            .spread(function (itemRecord, created) {
                masterItem.id = itemRecord.id;
                models.MasterItem.update(masterItem.dataValues, {where: {id: masterItem.id}})
                    .then(function (count) {
                        if (count > 0 && masterItem.medVendor) {
                            var ids = masterItem.medVendor.split(",");
                            models.ItemCatalog.destroy({where: {itemId: masterItem.id}}).then(function (num) {
                                var cats = [];
                                for (var i = 0; i < ids.length; i++) {
                                    var cat = models.ItemCatalog.build({
                                        itemId: masterItem.id,
                                        vendorId: ids[i]
                                    });
                                    cats.push(cat);
                                }
                                if (cats.length > 0) {
                                    var catpromises = [];

                                    cats.forEach(function (cat) {
                                        catpromises.push(cat.save());
                                    });

                                    models.sequelize.Promise.all(catpromises).then(function (vals) {
                                        transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp syc: " + JSON.stringify(masterItem));
                                        return cats.length;
                                    }).catch(function (saveErr) {
                                        logger.error(saveErr);
                                        return -1;
                                    });
                                }
                            }).catch(function (delerr) {
                                logger.error(delerr);
                                return -1;
                            });
                        } else {
                            transactionLogger.info("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                            return count;
                        }
                    }).catch(function (upderr) {
                    transactionLogger.error(upderr);
                    return -1;
                });
            }).catch(function (err) {
                logger.error(err);
                return -1;
            });
    }


    function bulkUpsertMasterItems(masterItems) {
        var promises = [];

        masterItems.forEach(function (masterItem) {
            promises.push(upsertMasterItem(masterItem));
        });

        return models.sequelize.Promise.all(promises);
    }

    function upsertClinicProfile(clinicProfile) {
        return models.ClinicProfile.findOrCreate({where: {clinicId: clinicProfile.clinicId}, raw: true})
            .spread(function (clinicRecord, created) {
                clinicProfile.id = clinicRecord.id;
                models.ClinicProfile.update(clinicProfile.dataValues, {where: {id: clinicProfile.id}})
                    .then(function (count) {
                        if (count > 0 && clinicProfile.vendorList) {
                            var ids = clinicProfile.vendorList.split(",");
                            models.ClinicCatalog.destroy({where: {clinicId: clinicProfile.id}}).then(function (num) {
                                var cats = [];
                                for (var i = 0; i < ids.length; i++) {
                                    var cat = models.ClinicCatalog.build({
                                        clinicId: clinicProfile.id,
                                        vendorId: ids[i]
                                    });
                                    cats.push(cat);
                                }
                                if (cats.length > 0) {
                                    var catpromises = [];

                                    cats.forEach(function (cat) {
                                        catpromises.push(cat.save());
                                    });

                                    models.sequelize.Promise.all(catpromises).then(function (vals) {
                                        transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                                        return cats.length;
                                    }).catch(function (saveErr) {
                                        logger.error(saveErr);
                                        return -1;
                                    });
                                }
                            })
                        } else {
                            transactionLogger.info("Clinic profile " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(clinicProfile));
                            return count;
                        }
                    }).catch(function (upderr) {
                    logger.error(upderr);
                    return -1;
                });
            }).catch(function (err) {
                logger.error(err);
                return -1;
            });
    }

    function bulkUpsertClinicProfiles(clinicProfiles) {
        var promises = [];

        clinicProfiles.forEach(function (clinicProfile) {
            promises.push(upsertClinicProfile(clinicProfile));
        });

        return models.sequelize.Promise.all(promises);
    }

};
