/**
 * Created by John on 3/7/2016.
 */
var moment = require('moment');
var ods = require("./fres-basic-ods");
var winston = require('winston');
var logger = winston.loggers.get('default');

function fromJson2Xl(params) {
    //if (!callback || typeof callback !== 'function') {
    //    throw new Error('Callback is required');
    //}

    return new Promise(function (resolve, reject) {
        checkParams(params).then(function (good) {

            var m = moment().format("YYYYMMDD_hhmm");
            var filepath = "components/public/uploads/"; //no longer used to generate the file
            var filePrefix = (params.prefix) ? params.prefix : "excel";
            var sheetName = filePrefix + "_" + m;
            var fileName = sheetName + '.xlsx';

            var rows = [];

            rows.push(params.fields);

            rows = rows.concat(params.data);

            resolve(ods.toBuffer(rows));
        }, function(err){
            reject(err);
        });
    });
}

/**
 * Check passing params
 *
 * @param {Object} params Function parameters containing data, fields,
 * delimiter, default value, mark quotes and hasCSVColumnTitle
 * @param {Function} callback Callback function returning error when invalid field is found
 */
function checkParams(params) {
    // if data is an Object, not in array [{}], then just create 1 item array.
    // So from now all data in array of object format.
    return new Promise(function (resolve, reject) {
        if (!Array.isArray(params.data)) {
            var ar = [];
            ar[0] = params.data;
            params.data = ar;
        }

        if (!params.fields && params.data && params.data.length) {
            var firstData = params.data[0];

            if (typeof firstData === 'object' && firstData !== null) {
                params.fields = Object.keys(firstData);
            } else {
                logger.error('params should be a valid object.');
                reject(false);
            }
        }

        //#check fieldNames
        if (params.fieldNames && params.fieldNames.length !== params.fields.length) {
            logger.error('fieldNames and fields should be of the same length, if fieldNames is provided.');
            reject(false);
        }

        params.fieldNames = params.fieldNames || params.fields;

        //#check delimiter
        params.del = params.del || ',';

        //#check end of line character
        params.eol = params.eol || '';

        //#check quotation mark
        params.quotes = typeof params.quotes === 'string' ? params.quotes : '"';

        //#check nested option
        params.nested = params.nested || false;

        //#check default value
        params.defaultValue = params.defaultValue;

        //#check hasCSVColumnTitle, if it is not explicitly set to false then true.
        if (params.hasCSVColumnTitle !== false) {
            params.hasCSVColumnTitle = true;
        }

        resolve(true);
    });
}

module.exports = {
    fromJson2Xl: fromJson2Xl
}