"use strict";

var moment = require('moment');
var winston = require('winston');
var logger = winston.loggers.get('default');

module.exports = function(sequelize, DataTypes) {
    var CountData = sequelize.define('CountData', {
            id: {
                type: DataTypes.INTEGER,
                field: 'id',
                primaryKey: true,
                autoIncrement: true
            },
            devicePk: {
                type: DataTypes.INTEGER,
                field: 'device_pk'
            },
            deviceId: {
                type: DataTypes.STRING,
                field: 'device_id'
            },
            timestamp: {
                type: DataTypes.DATE,
                field: 'time_stamp',
                set: function(val) {
                    if (val == null) {
                        this.setDataValue('timestamp', null);
                    } else {
                        try {
                            var m = moment(val, "YYYY-MM-DDTHH:mm:ss.SSSZ");
                            if (!m.isValid()) {
                                m = moment(val, "YYYY-MM-DD HH:mm:ss.SSS");
                                if (!m.isValid()) {
                                    var mom = moment();
                                    logger.warn("Invalid Timestamp: " + val);
                                    this.setDataValue('timestamp', mom);
                                } else {
                                    this.setDataValue('timestamp', m)
                                }
                            } else {
                                this.setDataValue('timestamp', m)
                            }
                        } catch (e) {
                            var mo = moment();
                            this.setDataValue('timestamp', mo);
                        }
                    }
                }
            },
            clinicId: {
                type: DataTypes.STRING,
                field: 'clinic_id'
            },
            sapMaterial: {
                type: DataTypes.STRING,
                field: 'sap_material'
            },
            vendorMaterial: {
                type: DataTypes.STRING,
                field: 'vendor_material'
            },
            description: {
                type: DataTypes.STRING,
                field: 'material_description'
            },
            count: {
                type: DataTypes.INTEGER,
                field: 'count'
            },
            countType: {
                type: DataTypes.STRING,
                field: 'count_type'
            },
            catType: {
                type: DataTypes.STRING,
                field: 'cat_type'
            },
            cntInd: {
                type: DataTypes.BOOLEAN,
                field: 'cnt_ind'
            },
            dateModified: {
                type: DataTypes.DATE,
                field: 'date_modified'
            }
        },
        {
            timestamps: false,
            tableName: 'tbl_count_data',
            indexes: [
                // Create a unique index on device timestamp
                {
                    unique: true,
                    fields: ['deviceId','timestamp']
                },
            ]
        }
    );
    return CountData;
};
