/**
 * Created by will on 10/2/15.
 */
angular.module('app').controller('MasterItemCtrl', [
    '$scope',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {

        var requestData = {};
        var existingRow = {};

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        }

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.masterItemCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
        };
        $scope.gridOptions.totalItems = 0;

        function getPage() {
            return $http.post('/api/v2/getMasterItem', requestData)
                .then(function(res){
                    $scope.gridOptions.data = null;
                    $scope.gridOptions.data = res.data;
                },function(err) {
                    $scope.gridOptions.data = null;
                    console.log("NO REFRESH!!!!" + err);
                });

        }

        $scope.isDisabled = false;
        $scope.showSapVerifyAlert = false;
        $scope.showVendorVerifyAlert = false;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function openEditDialog(row) {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            $scope.editVendor = angular.copy($scope.vendorList);
            //Mark a new "Checked" field, based on vendorList string
            if (row != null) {
                //edit mode
                for (var v = 0; v < $scope.editVendor.length; v++) {
                    $scope.editVendor[v].checked = $scope.isCheckboxChecked($scope.editVendor[v].vendorId);
                }
            } else {
                //add mode
                for (var v = 0; v < $scope.editVendor.length; v++) {
                    $scope.editVendor[v].checked = $scope.editVendor[v].isDefault;
                }
            }
            //$scope.editVendor = angular.copy($scope.vendorList);
            ngDialog.open({
                template: '/masterItem/masterItemMaintenance.html',
                showClose: true,
                className: "ngdialog-theme-default masterItem",
                scope: $scope
            });
        }

        function getVendors(row) {
            if ($scope.vendorList == null) {
                console.log("getting Vendors");
                return $http.post('/api/getVendorList', null).then(function (res) {
                    $scope.vendorList = res.data;
                    openEditDialog(row);
                }, function (err) {
                    $scope.vendorList = null;
                });
            } else {
                openEditDialog(row);
            }
        }



        $scope.addRowModal = function (row) {
            $scope.editRow = { cntInd: false, countType: "", medVendor: "" };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
                if (JSON.stringify($scope.editRow.medVendor) == "null") $scope.editRow.medVendor = "";
            }
            getVendors(row);
      };

        $scope.deleteRow = function(row) {
            if (row != null) {
                $scope.loading.busyPromise = $http.post('/api/deleteMasterItem', row.entity).then(function(res) {
                    getPage();
                });
            }
        };


        $scope.saveMasterItem = function(row, vendors) {
            $scope.editRow.dateModified = new Date();
            //Before saving, build "new" vendorList string
            $scope.editRow.medVendor = "";
            var firstEntry = true;
            for (var v = 0; v < vendors.length; v++ ) {
                if (vendors[v].checked) {
                    if (!firstEntry) {
                        $scope.editRow.medVendor += ",";
                    }
                    $scope.editRow.medVendor += vendors[v].id;
                    firstEntry = false;
                }
            }
            $scope.savingData.busyPromise = $http.post('/api/v2/saveMasterItem', $scope.editRow)
                .then(function(res) {
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                }, function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                });
        };

        $scope.verifyMasterItem = function() {
            console.log("verifying");
            var row = {};
            row.vendorMaterial = $scope.editRow.vendorMaterial;
            row.sapMaterial = $scope.editRow.sapMaterial;
            //query for the combo
            $http.post('/api/v2/getMasterItem', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];
                        if (existingRow.sapMaterial != null && existingRow.sapMaterial == row.sapMaterial) {
                            $scope.showSapVerifyAlert = true;
                        } else if (existingRow.vendorMaterial != null && existingRow.vendorMaterial == row.vendorMaterial) {
                            $scope.showVendorVerifyAlert = true;
                        }
                    } else {
                        $scope.isDisabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Master Item: " + err);
                    $scope.closeDialog();
                })
        };

        $scope.editExistingRow = function() {
            $scope.editRow = existingRow;
            $scope.showSapVerifyAlert = false;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = true;
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showSapVerifyAlert = false;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
        }

        $scope.exportData = function() {
            var request = requestData;
            //delete request.limit;
            //delete request.offset;

            $scope.busy.busyPromise = getMasterItemPromise();
        };

        $scope.isCheckboxChecked = function (vendorName) {
            var rtn = false;
            if (vendorName == "-BLANK-") {
                rtn = ($scope.editRow.medVendor.startsWith(",") || $scope.editRow.medVendor.endsWith(",") || $scope.editRow.medVendor.indexOf(",,") > -1 ) ;
            }
            else {
                rtn = ( $scope.editRow.medVendor.trim() == vendorName );
                rtn = rtn || ($scope.editRow.medVendor.endsWith(", " + vendorName));
                rtn = rtn || ($scope.editRow.medVendor.startsWith(vendorName + ","));
                rtn = rtn || ($scope.editRow.medVendor.indexOf(", " + vendorName + ",") > -1);
            }
            return rtn;
        };

        function getMasterItemPromise() {
            return $http.post('/api/v2/getMasterItemFile/', requestData).then(function(res) {
                var file = new Blob([ res.data ], {
                    type : 'text/tab-separated-values;charset=utf-8a'
                });
                var m = moment().format("YYYYMMDD_hhmm");
                FileSaver.saveAs(file, 'M3Extract_' + m + '.csv');
                //var b64 = res.data;
                //blobUtil.base64StringToBlob(b64).then(function (blob) {
                    // success
                //    var m = moment().format("YYYYMMDD_hhmm");
                //    FileSaver.saveAs(blob, 'M3Extract_' + m + '.ods');
                //}).catch(function (err) {
                    // error
                //    console.log(err);
                //});


            });
        }

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function cleanVar(field) {
            if (field == "") field = null;
            return field;
        }

    }
]);