/**
 * Created by John on 4/28/2017.
 */
angular.module('app').controller('MessageCtrl', [
    '$scope',
    '$rootScope',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',
    '$window',

    function($scope, $rootScope, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob, $window) {

        var existingRow = {};

        $scope.setFocus = function(id) {
            $timeout(function() {
                var fld = $window.document.getElementById(id);
                if (fld) fld.focus();
            },10);
        };

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.messageCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
            $scope.gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
                onSortChanged(sortColumns);
            });
            $scope.gridApi.pagination.on.paginationChanged($scope, function(newPage, newSize) {
                requestData.offset = (newPage - 1) * newSize;
                requestData.limit = newSize;
                getPage();
            });
            $scope.gridApi.core.on.filterChanged($scope, onFilterChanged)
        };
        $scope.gridOptions.totalItems = 0;

        var requestData = {
            offset: 0,
            limit: 50,
            order: "'timestamp' DESC"
        };

        function getPage() {
            $http.post('/api/getMessagesTotal', requestData).then(function(res) {
                $scope.gridOptions.totalItems = res.data;
            });
            return $http.post('/api/getMessages', requestData)
                .then(function(res){
                    $scope.gridOptions.data = null;
                    $scope.gridOptions.data = res.data;
                }).catch(function(err) {
                    $scope.gridOptions.data = null;
                    console.log("NO REFRESH!!!!" + JSON.stringify(err));
                });

        }

        $scope.isDisabled = false;
        $scope.loading.busyPromise = getPage();
        //getClinics(null);

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function onFilterChanged() {
            var grid = $scope.gridApi.grid;

            for (var i = 0; i < grid.columns.length; i++) {
                var column = grid.columns[i];
                if (column.filters[0].term !== null && column.filters[0].term !== "") {
                    if (requestData["where"] == null) {
                        requestData["where"] = {};
                    }
                    requestData["where"][column.field] = column.filters[0].term;

                } else {
                    if (requestData["where"] !== null && requestData["where"][column.field] !== null) {
                        delete requestData["where"][column.field];
                    }
                }
            }
            getPage();
        }

        $scope.forceFilter = function() {
            return onFilterChanged();
        };

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.timestamp == null) {
                requestData.where.timestamp = {};
            }

            requestData.where.timestamp['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.timestamp == null) {
                requestData.where.timestamp = {};
            }

            requestData.where.timestamp['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");getPage();
        }

        $scope.isCheckboxChecked = function (clinicId) {
            var rtn = false;
            for (var r=0; r < $scope.editRow.recipients.length; r++) {
                var recip = "" + $scope.editRow.recipients[r].clinic.clinicId;
                rtn = rtn || ( recip.trim() === clinicId );
                if (rtn) break;
            }
            return rtn;
        };

        /************************ MODAL METHODS *************************/
        $scope.filtering = false;
        $scope.clinicQuery="";
        $scope.oldQuery = $scope.clinicQuery;
        $scope.showHint=true;
        $scope.filteredClinics=[];
        $scope.maxMessageSize=350;

        //get the correct starting array to filter
        function arrayFactory(newval) {
            //first, if we have no previous filter, or we want all entries, use the original list
            var arr = ($scope.filteredClinics.length === 0 || !newval) ? $scope.editClinics : $scope.filteredClinics;
            //if we are using the filtered list, make sure it's a filter continuation and not a filter replace
            if ($scope.filteredClinics.length > 0 && newval) {
                if ($scope.oldQuery)
                    arr = (newval.startsWith($scope.oldQuery) ? $scope.filteredClinics : $scope.editClinics);
            }
            return arr;
        }

        //actual filter logic - more efficient than using a built-in filter
        $scope.filterAllClinics = function(newval) {
            if (!newval) {
                $scope.showHint = ($scope.editClinics.length > 0);
                $scope.filteredClinics = [];
                $scope.setFocus('filtered');
                $scope.oldQuery = newval;
                if ($scope.showHint && $scope.editClinics.length < 100) {
                    $scope.showHint = false;
                    $scope.filteredClinics = angular.copy($scope.editClinics);
                }
            }
            else {
                $scope.filtering = true;
                $timeout(function() {
                    var filtered = [];
                    newval = newval.toUpperCase();
                    var arr = arrayFactory(newval);
                    $scope.oldQuery = newval;
                    for (var c = 0; c < arr.length; c++) {
                        var item = arr[c];
                        if (item.clinicId.toUpperCase().indexOf(newval) !== -1) {
                            filtered.push(item);
                        }
                    }
                    $scope.filteredClinics = filtered;
                    $scope.showHint = false;
                    $scope.filtering = false;
                    $scope.setFocus('filtered');
                }, 10);
            }
        };

        //called by UI
        $scope.addAll = function(clinicQuery) {
            $scope.filtering = true;
            $timeout(ctrlAddAll, 10, true, clinicQuery);
        };

        //works in timeout to allow UI thread to refresh
        function ctrlAddAll(clinicQuery) {
            var result = [];
            if (!clinicQuery || clinicQuery === "") {
                result = $scope.selectedClinics.concat($scope.editClinics);
                $scope.editClinics = [];
                $scope.selectedClinics = result;
                $scope.filteredClinics = [];
            } else {
                //move the filtered pane to the selected pane
                result = $scope.selectedClinics.concat($scope.filteredClinics);
                $scope.filteredClinics = [];
                $scope.selectedClinics = result;
                //now keep the source data in sync
                $timeout(function() {
                    clinicQuery = clinicQuery.toLowerCase();
                    var i=$scope.editClinics.length;
                    while (i--) {
                        if (i < 0) break;
                        var item = $scope.editClinics[i];
                        if (item.clinicId.toLowerCase().indexOf(clinicQuery) !== -1) {
                            $scope.editClinics.splice(i,1);
                        }
                    }
                }, 10);
            }
            $scope.filtering = false;
            $scope.showHint = false; //hide the hint even though the left pane is empty
        }

        //called by UI
        $scope.removeAll = function(clinicQuery) {
            $scope.filtering = true;
            $timeout(ctrlRemoveAll, 10, true, clinicQuery);
        };

        //works in timeout to allow UI thread to refresh
        function ctrlRemoveAll(clinicQuery) {
            var result = $scope.editClinics.concat($scope.selectedClinics);
            $scope.selectedClinics = [];
            $scope.editClinics = result;
            $scope.filteredClinics = [];
            $scope.filtering = false;
            if (clinicQuery) {
                $scope.filterAllClinics(clinicQuery);
            } else {
                $scope.showHint = true;
            }
        }

        //move an item from the left pane to the right pane
        $scope.addSelected = function(idx, selected) {
            var selectedList = [];
            if (!selected) selected = idx;
            if (Array.isArray(selected)) {
                selectedList = selected;
            } else {
                selectedList.push(selected);
            }

            var test = $scope.filteredClinics[idx];
            if (test && test.clinicId === selected) { //should always be true
                $scope.selectedClinics.push(test);
                $scope.filteredClinics.splice(idx,1);
            }
            $timeout(function() {
                for (var s = 0; s < selectedList.length; s++) {
                    var item = selectedList[s];
                    var c = $scope.editClinics.length;
                    while (c--) {
                        if (c < 0) break;
                        var clinic = $scope.editClinics[c];
                        if (clinic.clinicId === item) {
                            $scope.editClinics.splice(c, 1);
                            break;
                        }
                    }
                }
            },10);
        };
        //move one item from the right pane to the left pane
        $scope.removeSelected = function(idx, removed, clinicQuery) {
            var test = $scope.selectedClinics[idx];
            if (test && test.clinicId === removed) { //should always be true
                $scope.editClinics.push($scope.selectedClinics[idx]);
                $scope.filteredClinics.push($scope.selectedClinics[idx]);
                $scope.selectedClinics.splice(idx, 1);
            }
            if (clinicQuery) $scope.filterAllClinics(clinicQuery);
        };

        $scope.showRemaining = function(elem, newval) {
            var element = $window.document.getElementById(elem);
            if (element) {
                element.textContent = "" + ((!newval) ? 0 : newval.length) + "/" + $scope.maxMessageSize;
            }
        };


        //OPen the modal, passing an empty or selected Message row
        $scope.addRowModal = function (row) {
            $scope.editRow = { };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
            }
            $scope.editRow.createdByUserId = $rootScope.user.id;
            getClinics(row);
            //openEditDialog(row);
        };

        $scope.deleteRow = function(row) {
            if (row != null) {
                $scope.loading.busyPromise = $http.post('/api/v2/deleteMessage', row.entity).then(function(res) {
                    getPage();
                });
            }
        };

        //pull the list of clinics from the db
        function getClinics(row) {
            $scope.busy.busyPromise = new Promise(function(resolve,reject){
                //if ($scope.clinicList == null) {
                console.log("getting Clinics");
                $http.post('/api/getAllClinics', null).then(function (res) {
                    $scope.clinicList = res.data;
                    openEditDialog(row, resolve);
                    //resolve(row);
                }).catch(function (err) {
                    $scope.clinicList = null;
                    console.log(err);
                    reject(err);
                });
            // } else {
            //     //openEditDialog(row);
            //     $scope.loading.busyPromise = getPage();
            // }
            });
        }

        function openEditDialog(row, resolve) {

            //Using editClinics as a bindable Object in the modal.... cloning the original Clinic list

            $scope.editClinics = $scope.clinicList;
            $scope.selectedClinics = [];
            //Mark a new "Checked" field, based on vendorList string
            if (row != null) {
                $scope.editRow.isRecurring = (row.entity.keepAliveStart !== null);
                //edit mode
                var v = $scope.editClinics.length;
                while (v--) {
                    if (v < 0) break;
                    if ($scope.isCheckboxChecked($scope.editClinics[v].clinicId)) {
                        $scope.selectedClinics.push($scope.editClinics[v]);
                        $scope.editClinics.splice(v,1);
                    }
                }
            }
            //$scope.editVendor = angular.copy($scope.vendorList);
            ngDialog.open({
                template: '/message/messageMaintenance.html',
                showClose: true,
                className: "ngdialog-theme-default messages",
                scope: $scope,
                onOpenCallback: function() {
                    $scope.showRemaining("remaining", $scope.editRow.message);
                    resolve();
                }
            });

        }

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.isDisabled = false;
        };

        $scope.saveMessage = function(row, clinics) {
            $scope.editRow.Recipients = clinics;
            if (!$scope.editRow.isRecurring) {
                $scope.editRow.keepAliveStart = null;
                $scope.editRow.keepAliveEnd = null;
            }
            $scope.savingData.busyPromise = $http.post('/api/v2/saveMessage', $scope.editRow)
                .then(function(res) {
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                }, function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                });
        };
    }
]);