/**
 * Created by John on 7/21/2016.
 */
angular.module('app').controller('VendorCtrl', [
    '$scope',
    '$window',
    '$http',
    'uiGridConstants',
    'Constants',
    'ngDialog',
    '$timeout',
    'FileSaver',
    'Blob',

    function($scope, $window, $http, uiGridConstants, Constants, ngDialog, $timeout, FileSaver, Blob) {

        var requestData = {};
        var existingRow = {};

        $scope.loading = {
            message: 'Loading Data...',
            backdrop: true,
            busyPromise: null
        }

        $scope.savingData = {
            message: 'Saving Data...',
            backdrop: true,
            busyPromise: null
        };

        $scope.busy = {
            message: 'Please wait...',
            backdrop: true,
            busyPromise: null
        };

        $scope.gridOptions = Constants.getDefaultGridOptions();
        $scope.gridOptions.columnDefs = Constants.vendorCols;
        $scope.gridOptions.onRegisterApi = function(gridApi) {
            $scope.gridApi = gridApi;
        };
        $scope.gridOptions.totalItems = 0;

        function getPage() {
            return $http.post('/api/getVendorList', requestData)
                .then(function(res){
                    $scope.gridOptions.data = null;
                    $scope.gridOptions.data = res.data;
                },function(err) {
                    $scope.gridOptions.data = null;
                    console.log("NO REFRESH!!!!" + err);
                });

        }

        $scope.isDisabled = false;
        $scope.showVendorVerifyAlert = false;

        $scope.loading.busyPromise = getPage();

        $scope.digits = {};

        //initializing date picker
        $scope.dateDbA = moment();
        $scope.dateDbB = moment();

        $scope.optionsDbA = {
            callback: fromUpdated
        };
        $scope.optionsDbB = {
            callback: toUpdated
        };
        $scope.menuOpened = false;

        $scope.toggleMenu = function(event) {
            $scope.menuOpened = !($scope.menuOpened);
            event.stopPropagation();
        };

        $scope.closeThis = function() {
            if ($scope.menuOpened) {
                $scope.menuOpened = false;
            }
        };

        function openEditDialog() {
            //Using editVendor as a bindable Object in the modal.... cloning the original Vendor list
            //$scope.editVendor = angular.copy($scope.vendorList);
            ngDialog.open({
                template: '/vendor/vendorMaintenance.html',
                showClose: true,
                scope: $scope
            });
        }

        $scope.addRowModal = function (row) {
            $scope.editRow = { isDefault: false };
            if (row != null) {
                $scope.editRow = findById($scope.gridOptions.data, row.entity.id);
            }
            openEditDialog();
        };

        $scope.deleteRow = function(row) {
            if (row != null) {
                var doDelete = $window.confirm("Are you sure?");
                if (doDelete) {
                    $scope.loading.busyPromise = $http.post('/api/deleteVendor', row.entity).then(function (res) {
                        $scope.loading.busyPromise = getPage();
                    });
                }
            }
        };


        $scope.saveVendor = function(row) {
            $scope.editRow.dateModified = new Date();
            $scope.savingData.busyPromise = $http.post('/api/saveVendor', $scope.editRow)
                .then(function(res) {
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                }, function(err) {
                    console.log("error: " + err);
                    $scope.closeDialog();
                    $scope.loading.busyPromise = getPage();
                });
        };

        $scope.verifyVendor = function() {
            console.log("verifying");
            var row = {};
            row.vendorId = $scope.editRow.vendorId;
            //query for the combo
            $http.post('/api/getVendor', row)
                .then(function(res) {
                    if (res.data[0] != null) {
                        existingRow = res.data[0];
                        if (existingRow.vendorId != null && existingRow.vendorId == row.vendorId) {
                            $scope.showVendorVerifyAlert = true;
                        }
                    } else {
                        $scope.isDisabled = true;
                    }
                }, function(err) {
                    console.log("Error checking Master Item: " + err);
                    $scope.closeDialog();
                })
        };

        $scope.editExistingRow = function() {
            $scope.editRow = existingRow;
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = true;
        };

        $scope.closeDialog = function(){
            ngDialog.close();
            $scope.showVendorVerifyAlert = false;
            $scope.isDisabled = false;
        };

        function findById(array, value) {
            for (var i = 0; i < array.length; i++) {
                if (array[i]['id'] === value) {
                    return array[i];
                }
            }
            return null;
        }

        function fromUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$gt'] = date.startOf('day');
            $scope.fromDate = date.format("MM-DD-YYYY");
            getPage();
        }

        function toUpdated(date) {
            if (requestData.where == null) {
                requestData.where = {};
            }
            if (requestData.where.dateModified == null) {
                requestData.where.dateModified = {};
            }

            requestData.where.dateModified['$lt'] = date.endOf('day');
            $scope.toDate = date.format("MM-DD-YYYY");
            getPage();
        }

    }
]);