"use strict";
var express = require('express');
var passport = require('passport');

var app = express();

//make Sequelize a singleton
app.set('models', require('./components/models'));
var models = app.get('models');

var port = process.env.PORT || 3414;

var env         = process.env.NODE_ENV || "dev";
var config      = require("./components/config/config.json")[env];

//initialize logging
require('./components/config/logging')(app);

var winston = require('winston');
var logger = winston.loggers.get('default');

//initialize database
//set updated to true, and sequelize will sync and update the tables, otherwise avoid
var updated = process.env.UPDATED || 'false';
var createData = process.env.CREATE_DATA || 'false';
var createType = process.env.CREATE_TYPE || null;

logger.debug("updated: " + updated);
logger.debug("create: " + createData);

if (updated == 'true') {
    logger.debug('updating tables');

    models.sequelize.sync({logging: console.log}).then(function() {
        if (createData == 'true') {
            logger.debug('creating mock data');

            require('./components/config/mockdata')(app, models, createType);
        }

        app.listen(port, '0.0.0.0');
        logger.info("Fresenius Console API Server started on port " + port);
    });
} else {
    if (createData == 'true') {
        require('./components/config/mockdata')(app, models, createType);
    }
    app.listen(port, '0.0.0.0');
    logger.info("Fresenius Console API Server started on port " + port);
}


//initialize passport
require('./components/config/passport')(passport, models);

//initialize express
require('./components/config/express')(app, passport);

//initialize routes
require('./routes/api')(app, passport, models);
//require('./routes/test')(app, passport, models);
require('./routes/web')(app, passport, models);

//start watching Master Item file
require('./components/config/filewatcher')(app);


module.exports = app;
